(** A temporary holds a variable which should be in a register
   during real execution *)
module Temp:
  sig
    type temp
	  (** abstact type for "something that holds a value" *)

    val new_temp: unit -> temp (** create a unique temp *)
    val named_temp: string -> temp
          (** create a new UNIQUE labeled temp (e.g. for pretty-printing purpose) *)

    val string: temp -> string
	  (** get the label of a temp, given by the [named_temp] function
	     or created on the fly. The same string is always returned for a
	     given temp *)
  end


(** A label is the name of an address in memory. It can refer to an instruction
   (target of jumps) or a string for example *)
module Label:
  sig
    type label = Symbol.symbol
    val new_label: unit -> label
    val unique_named_label: string -> label (* unique id containing string *)
    val named_label: string -> label        (* exactly the string *)
    val name: label -> string               (* unique generic label *)
  end
