type ty =
  | INT
      (** integer *)

  | BOOL
      (** boolean *)

  | STRING
      (** string *)

  | RECORD of (Symbol.symbol * ty) list * int
      (** [RECORD ([(f,t);...],s)] is the type of a record
	which contains the field [f] of type [t]. [s] is
	the memory size of the record *)

  | ARRAY of ty * int * int * int
      (** [ARRAY (t,l,u,s)] is the type of an array of elements of type [t]
	 from [l] to [u] where each element has size [s] *)

  | PROC of (ty * bool) list
      (** [PROC ([(t,b) ;...])] is the the type of a procedure whose first argument
	is of type [t] and [b] is true if this argument escapes *)

  | CONST of ty
      (** constant *)

  | POINTER of ty
      (** pointer *)

  | VOID

  | SELF of ty ref
      (** to allow pointers on the same type *)

  | SAMEAS of Symbol.symbol
      (** same type as the type refered by the symbol
	(useful for self or mutually references) *)
  | ANY
      (** mean any type, kind of polymorphism *)

val equal : ty -> ty -> bool
  (** [equal tenv t1 t2] returns [true] if and only if [t1] and [t2]
    represent the same type *)

val to_string : ty -> string

val size : ty -> int
  (** [size t]  give the size of an element of type [t] *)
